"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cursorpagination import CursorPagination, CursorPaginationTypedDict
from .event import Event, EventTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceWithCursorPaginationEventTypedDict(TypedDict):
    items: List[EventTypedDict]
    pagination: CursorPaginationTypedDict


class ListResourceWithCursorPaginationEvent(BaseModel):
    items: List[Event]

    pagination: CursorPagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class CustomerPaymentMethodCreateRequiresActionResponseTypedDict(TypedDict):
    client_secret: str
    status: Literal["requires_action"]


class CustomerPaymentMethodCreateRequiresActionResponse(BaseModel):
    client_secret: str

    STATUS: Annotated[
        Annotated[
            Literal["requires_action"],
            AfterValidator(validate_const("requires_action")),
        ],
        pydantic.Field(alias="status"),
    ] = "requires_action"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerseat import CustomerSeat, CustomerSeatTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class WebhookCustomerSeatClaimedPayloadTypedDict(TypedDict):
    r"""Sent when a customer seat is claimed.

    This event is triggered when a customer accepts the seat invitation and claims their access.
    """

    timestamp: datetime
    data: CustomerSeatTypedDict
    type: Literal["customer_seat.claimed"]


class WebhookCustomerSeatClaimedPayload(BaseModel):
    r"""Sent when a customer seat is claimed.

    This event is triggered when a customer accepts the seat invitation and claims their access.
    """

    timestamp: datetime

    data: CustomerSeat

    TYPE: Annotated[
        Annotated[
            Literal["customer_seat.claimed"],
            AfterValidator(validate_const("customer_seat.claimed")),
        ],
        pydantic.Field(alias="type"),
    ] = "customer_seat.claimed"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class OrderRefundedMetadataTypedDict(TypedDict):
    order_id: str
    refunded_amount: int
    currency: str


class OrderRefundedMetadata(BaseModel):
    order_id: str

    refunded_amount: int

    currency: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .costmetadata_output import CostMetadataOutput, CostMetadataOutputTypedDict
from .llmmetadata import LLMMetadata, LLMMetadataTypedDict
from typing import Union
from typing_extensions import TypeAliasType


EventMetadataOutputTypedDict = TypeAliasType(
    "EventMetadataOutputTypedDict",
    Union[CostMetadataOutputTypedDict, LLMMetadataTypedDict, str, int, float, bool],
)


EventMetadataOutput = TypeAliasType(
    "EventMetadataOutput", Union[CostMetadataOutput, LLMMetadata, str, int, float, bool]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class SubscriptionBillingPeriodUpdatedMetadataTypedDict(TypedDict):
    subscription_id: str
    old_period_end: str
    new_period_end: str


class SubscriptionBillingPeriodUpdatedMetadata(BaseModel):
    subscription_id: str

    old_period_end: str

    new_period_end: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class SeatAssignTypedDict(TypedDict):
    subscription_id: NotRequired[Nullable[str]]
    r"""Subscription ID. Required if checkout_id and order_id are not provided."""
    checkout_id: NotRequired[Nullable[str]]
    r"""Checkout ID. Used to look up subscription or order from the checkout page."""
    order_id: NotRequired[Nullable[str]]
    r"""Order ID for one-time purchases. Required if subscription_id and checkout_id are not provided."""
    email: NotRequired[Nullable[str]]
    r"""Email of the customer to assign the seat to"""
    external_customer_id: NotRequired[Nullable[str]]
    r"""External customer ID for the seat assignment"""
    customer_id: NotRequired[Nullable[str]]
    r"""Customer ID for the seat assignment"""
    metadata: NotRequired[Nullable[Dict[str, Any]]]
    r"""Additional metadata for the seat (max 10 keys, 1KB total)"""
    immediate_claim: NotRequired[bool]
    r"""If true, the seat will be immediately claimed without sending an invitation email. API-only feature."""


class SeatAssign(BaseModel):
    subscription_id: OptionalNullable[str] = UNSET
    r"""Subscription ID. Required if checkout_id and order_id are not provided."""

    checkout_id: OptionalNullable[str] = UNSET
    r"""Checkout ID. Used to look up subscription or order from the checkout page."""

    order_id: OptionalNullable[str] = UNSET
    r"""Order ID for one-time purchases. Required if subscription_id and checkout_id are not provided."""

    email: OptionalNullable[str] = UNSET
    r"""Email of the customer to assign the seat to"""

    external_customer_id: OptionalNullable[str] = UNSET
    r"""External customer ID for the seat assignment"""

    customer_id: OptionalNullable[str] = UNSET
    r"""Customer ID for the seat assignment"""

    metadata: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Additional metadata for the seat (max 10 keys, 1KB total)"""

    immediate_claim: Optional[bool] = False
    r"""If true, the seat will be immediately claimed without sending an invitation email. API-only feature."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "subscription_id",
            "checkout_id",
            "order_id",
            "email",
            "external_customer_id",
            "customer_id",
            "metadata",
            "immediate_claim",
        ]
        nullable_fields = [
            "subscription_id",
            "checkout_id",
            "order_id",
            "email",
            "external_customer_id",
            "customer_id",
            "metadata",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Dict, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


SubscriptionCreateExternalCustomerMetadataTypedDict = TypeAliasType(
    "SubscriptionCreateExternalCustomerMetadataTypedDict", Union[str, int, float, bool]
)


SubscriptionCreateExternalCustomerMetadata = TypeAliasType(
    "SubscriptionCreateExternalCustomerMetadata", Union[str, int, float, bool]
)


class SubscriptionCreateExternalCustomerTypedDict(TypedDict):
    r"""Create a subscription for an existing customer identified by an external ID."""

    product_id: str
    r"""The ID of the recurring product to subscribe to. Must be a free product, otherwise the customer should go through a checkout flow."""
    external_customer_id: str
    r"""The ID of the customer in your system to create the subscription for. It must already exist in Polar."""
    metadata: NotRequired[
        Dict[str, SubscriptionCreateExternalCustomerMetadataTypedDict]
    ]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """


class SubscriptionCreateExternalCustomer(BaseModel):
    r"""Create a subscription for an existing customer identified by an external ID."""

    product_id: str
    r"""The ID of the recurring product to subscribe to. Must be a free product, otherwise the customer should go through a checkout flow."""

    external_customer_id: str
    r"""The ID of the customer in your system to create the subscription for. It must already exist in Polar."""

    metadata: Optional[Dict[str, SubscriptionCreateExternalCustomerMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CheckoutCreatedMetadataTypedDict(TypedDict):
    checkout_id: str
    checkout_status: str
    product_id: NotRequired[str]


class CheckoutCreatedMetadata(BaseModel):
    checkout_id: str

    checkout_status: str

    product_id: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerorderconfirmpayment import (
    CustomerOrderConfirmPayment,
    CustomerOrderConfirmPaymentTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import (
    FieldMetadata,
    PathParamMetadata,
    RequestMetadata,
    SecurityMetadata,
)
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CustomerPortalOrdersConfirmRetryPaymentSecurityTypedDict(TypedDict):
    customer_session: NotRequired[str]
    member_session: NotRequired[str]


class CustomerPortalOrdersConfirmRetryPaymentSecurity(BaseModel):
    customer_session: Annotated[
        Optional[str],
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="http",
                sub_type="bearer",
                field_name="Authorization",
            )
        ),
    ] = None

    member_session: Annotated[
        Optional[str],
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="http",
                sub_type="bearer",
                field_name="Authorization",
            )
        ),
    ] = None


class CustomerPortalOrdersConfirmRetryPaymentRequestTypedDict(TypedDict):
    id: str
    r"""The order ID."""
    customer_order_confirm_payment: CustomerOrderConfirmPaymentTypedDict


class CustomerPortalOrdersConfirmRetryPaymentRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The order ID."""

    customer_order_confirm_payment: Annotated[
        CustomerOrderConfirmPayment,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .listresource_benefitgrant_ import (
    ListResourceBenefitGrant,
    ListResourceBenefitGrantTypedDict,
)
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Callable, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


QueryParamCustomerIDFilterTypedDict = TypeAliasType(
    "QueryParamCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by customer."""


QueryParamCustomerIDFilter = TypeAliasType(
    "QueryParamCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by customer."""


MemberIDFilterTypedDict = TypeAliasType(
    "MemberIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by member."""


MemberIDFilter = TypeAliasType("MemberIDFilter", Union[str, List[str]])
r"""Filter by member."""


class BenefitsGrantsRequestTypedDict(TypedDict):
    id: str
    is_granted: NotRequired[Nullable[bool]]
    r"""Filter by granted status. If `true`, only granted benefits will be returned. If `false`, only revoked benefits will be returned."""
    customer_id: NotRequired[Nullable[QueryParamCustomerIDFilterTypedDict]]
    r"""Filter by customer."""
    member_id: NotRequired[Nullable[MemberIDFilterTypedDict]]
    r"""Filter by member."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""


class BenefitsGrantsRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    is_granted: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by granted status. If `true`, only granted benefits will be returned. If `false`, only revoked benefits will be returned."""

    customer_id: Annotated[
        OptionalNullable[QueryParamCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by customer."""

    member_id: Annotated[
        OptionalNullable[MemberIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by member."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["is_granted", "customer_id", "member_id", "page", "limit"]
        nullable_fields = ["is_granted", "customer_id", "member_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class BenefitsGrantsResponseTypedDict(TypedDict):
    result: ListResourceBenefitGrantTypedDict


class BenefitsGrantsResponse(BaseModel):
    next: Callable[[], Optional[BenefitsGrantsResponse]]

    result: ListResourceBenefitGrant



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class SeatClaimTypedDict(TypedDict):
    invitation_token: str
    r"""Invitation token to claim the seat"""


class SeatClaim(BaseModel):
    invitation_token: str
    r"""Invitation token to claim the seat"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpriceseattiers_input import (
    ProductPriceSeatTiersInput,
    ProductPriceSeatTiersInputTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ProductPriceSeatBasedCreateTypedDict(TypedDict):
    r"""Schema to create a seat-based price with volume-based tiers."""

    seat_tiers: ProductPriceSeatTiersInputTypedDict
    r"""List of pricing tiers for seat-based pricing.

    The minimum and maximum seat limits are derived from the tiers:
    - minimum_seats = first tier's min_seats
    - maximum_seats = last tier's max_seats (None for unlimited)
    """
    amount_type: Literal["seat_based"]
    price_currency: NotRequired[str]
    r"""The currency. Currently, only `usd` is supported."""


class ProductPriceSeatBasedCreate(BaseModel):
    r"""Schema to create a seat-based price with volume-based tiers."""

    seat_tiers: ProductPriceSeatTiersInput
    r"""List of pricing tiers for seat-based pricing.

    The minimum and maximum seat limits are derived from the tiers:
    - minimum_seats = first tier's min_seats
    - maximum_seats = last tier's max_seats (None for unlimited)
    """

    AMOUNT_TYPE: Annotated[
        Annotated[Literal["seat_based"], AfterValidator(validate_const("seat_based"))],
        pydantic.Field(alias="amount_type"),
    ] = "seat_based"

    price_currency: Optional[str] = "usd"
    r"""The currency. Currently, only `usd` is supported."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitdownloadables import BenefitDownloadables, BenefitDownloadablesTypedDict
from .benefitgrantdownloadablesproperties import (
    BenefitGrantDownloadablesProperties,
    BenefitGrantDownloadablesPropertiesTypedDict,
)
from .benefitgranterror import BenefitGrantError, BenefitGrantErrorTypedDict
from .customer import Customer, CustomerTypedDict
from .member import Member, MemberTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class BenefitGrantDownloadablesWebhookTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the grant."""
    is_granted: bool
    r"""Whether the benefit is granted."""
    is_revoked: bool
    r"""Whether the benefit is revoked."""
    subscription_id: Nullable[str]
    r"""The ID of the subscription that granted this benefit."""
    order_id: Nullable[str]
    r"""The ID of the order that granted this benefit."""
    customer_id: str
    r"""The ID of the customer concerned by this grant."""
    benefit_id: str
    r"""The ID of the benefit concerned by this grant."""
    customer: CustomerTypedDict
    r"""A customer in an organization."""
    benefit: BenefitDownloadablesTypedDict
    properties: BenefitGrantDownloadablesPropertiesTypedDict
    granted_at: NotRequired[Nullable[datetime]]
    r"""The timestamp when the benefit was granted. If `None`, the benefit is not granted."""
    revoked_at: NotRequired[Nullable[datetime]]
    r"""The timestamp when the benefit was revoked. If `None`, the benefit is not revoked."""
    member_id: NotRequired[Nullable[str]]
    r"""The ID of the member concerned by this grant."""
    error: NotRequired[Nullable[BenefitGrantErrorTypedDict]]
    r"""The error information if the benefit grant failed with an unrecoverable error."""
    member: NotRequired[Nullable[MemberTypedDict]]
    previous_properties: NotRequired[
        Nullable[BenefitGrantDownloadablesPropertiesTypedDict]
    ]


class BenefitGrantDownloadablesWebhook(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the grant."""

    is_granted: bool
    r"""Whether the benefit is granted."""

    is_revoked: bool
    r"""Whether the benefit is revoked."""

    subscription_id: Nullable[str]
    r"""The ID of the subscription that granted this benefit."""

    order_id: Nullable[str]
    r"""The ID of the order that granted this benefit."""

    customer_id: str
    r"""The ID of the customer concerned by this grant."""

    benefit_id: str
    r"""The ID of the benefit concerned by this grant."""

    customer: Customer
    r"""A customer in an organization."""

    benefit: BenefitDownloadables

    properties: BenefitGrantDownloadablesProperties

    granted_at: OptionalNullable[datetime] = UNSET
    r"""The timestamp when the benefit was granted. If `None`, the benefit is not granted."""

    revoked_at: OptionalNullable[datetime] = UNSET
    r"""The timestamp when the benefit was revoked. If `None`, the benefit is not revoked."""

    member_id: OptionalNullable[str] = UNSET
    r"""The ID of the member concerned by this grant."""

    error: OptionalNullable[BenefitGrantError] = UNSET
    r"""The error information if the benefit grant failed with an unrecoverable error."""

    member: OptionalNullable[Member] = UNSET

    previous_properties: OptionalNullable[BenefitGrantDownloadablesProperties] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "granted_at",
            "revoked_at",
            "member_id",
            "error",
            "member",
            "previous_properties",
        ]
        nullable_fields = [
            "modified_at",
            "granted_at",
            "revoked_at",
            "subscription_id",
            "order_id",
            "member_id",
            "error",
            "member",
            "previous_properties",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .metric import Metric, MetricTypedDict
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class MetricsTypedDict(TypedDict):
    orders: NotRequired[Nullable[MetricTypedDict]]
    revenue: NotRequired[Nullable[MetricTypedDict]]
    net_revenue: NotRequired[Nullable[MetricTypedDict]]
    cumulative_revenue: NotRequired[Nullable[MetricTypedDict]]
    net_cumulative_revenue: NotRequired[Nullable[MetricTypedDict]]
    costs: NotRequired[Nullable[MetricTypedDict]]
    cumulative_costs: NotRequired[Nullable[MetricTypedDict]]
    average_order_value: NotRequired[Nullable[MetricTypedDict]]
    net_average_order_value: NotRequired[Nullable[MetricTypedDict]]
    average_revenue_per_user: NotRequired[Nullable[MetricTypedDict]]
    cost_per_user: NotRequired[Nullable[MetricTypedDict]]
    active_user_by_event: NotRequired[Nullable[MetricTypedDict]]
    one_time_products: NotRequired[Nullable[MetricTypedDict]]
    one_time_products_revenue: NotRequired[Nullable[MetricTypedDict]]
    one_time_products_net_revenue: NotRequired[Nullable[MetricTypedDict]]
    new_subscriptions: NotRequired[Nullable[MetricTypedDict]]
    new_subscriptions_revenue: NotRequired[Nullable[MetricTypedDict]]
    new_subscriptions_net_revenue: NotRequired[Nullable[MetricTypedDict]]
    renewed_subscriptions: NotRequired[Nullable[MetricTypedDict]]
    renewed_subscriptions_revenue: NotRequired[Nullable[MetricTypedDict]]
    renewed_subscriptions_net_revenue: NotRequired[Nullable[MetricTypedDict]]
    active_subscriptions: NotRequired[Nullable[MetricTypedDict]]
    committed_subscriptions: NotRequired[Nullable[MetricTypedDict]]
    monthly_recurring_revenue: NotRequired[Nullable[MetricTypedDict]]
    committed_monthly_recurring_revenue: NotRequired[Nullable[MetricTypedDict]]
    checkouts: NotRequired[Nullable[MetricTypedDict]]
    succeeded_checkouts: NotRequired[Nullable[MetricTypedDict]]
    checkouts_conversion: NotRequired[Nullable[MetricTypedDict]]
    canceled_subscriptions: NotRequired[Nullable[MetricTypedDict]]
    canceled_subscriptions_customer_service: NotRequired[Nullable[MetricTypedDict]]
    canceled_subscriptions_low_quality: NotRequired[Nullable[MetricTypedDict]]
    canceled_subscriptions_missing_features: NotRequired[Nullable[MetricTypedDict]]
    canceled_subscriptions_switched_service: NotRequired[Nullable[MetricTypedDict]]
    canceled_subscriptions_too_complex: NotRequired[Nullable[MetricTypedDict]]
    canceled_subscriptions_too_expensive: NotRequired[Nullable[MetricTypedDict]]
    canceled_subscriptions_unused: NotRequired[Nullable[MetricTypedDict]]
    canceled_subscriptions_other: NotRequired[Nullable[MetricTypedDict]]
    churned_subscriptions: NotRequired[Nullable[MetricTypedDict]]
    churn_rate: NotRequired[Nullable[MetricTypedDict]]
    ltv: NotRequired[Nullable[MetricTypedDict]]
    gross_margin: NotRequired[Nullable[MetricTypedDict]]
    gross_margin_percentage: NotRequired[Nullable[MetricTypedDict]]
    cashflow: NotRequired[Nullable[MetricTypedDict]]


class Metrics(BaseModel):
    orders: OptionalNullable[Metric] = UNSET

    revenue: OptionalNullable[Metric] = UNSET

    net_revenue: OptionalNullable[Metric] = UNSET

    cumulative_revenue: OptionalNullable[Metric] = UNSET

    net_cumulative_revenue: OptionalNullable[Metric] = UNSET

    costs: OptionalNullable[Metric] = UNSET

    cumulative_costs: OptionalNullable[Metric] = UNSET

    average_order_value: OptionalNullable[Metric] = UNSET

    net_average_order_value: OptionalNullable[Metric] = UNSET

    average_revenue_per_user: OptionalNullable[Metric] = UNSET

    cost_per_user: OptionalNullable[Metric] = UNSET

    active_user_by_event: OptionalNullable[Metric] = UNSET

    one_time_products: OptionalNullable[Metric] = UNSET

    one_time_products_revenue: OptionalNullable[Metric] = UNSET

    one_time_products_net_revenue: OptionalNullable[Metric] = UNSET

    new_subscriptions: OptionalNullable[Metric] = UNSET

    new_subscriptions_revenue: OptionalNullable[Metric] = UNSET

    new_subscriptions_net_revenue: OptionalNullable[Metric] = UNSET

    renewed_subscriptions: OptionalNullable[Metric] = UNSET

    renewed_subscriptions_revenue: OptionalNullable[Metric] = UNSET

    renewed_subscriptions_net_revenue: OptionalNullable[Metric] = UNSET

    active_subscriptions: OptionalNullable[Metric] = UNSET

    committed_subscriptions: OptionalNullable[Metric] = UNSET

    monthly_recurring_revenue: OptionalNullable[Metric] = UNSET

    committed_monthly_recurring_revenue: OptionalNullable[Metric] = UNSET

    checkouts: OptionalNullable[Metric] = UNSET

    succeeded_checkouts: OptionalNullable[Metric] = UNSET

    checkouts_conversion: OptionalNullable[Metric] = UNSET

    canceled_subscriptions: OptionalNullable[Metric] = UNSET

    canceled_subscriptions_customer_service: OptionalNullable[Metric] = UNSET

    canceled_subscriptions_low_quality: OptionalNullable[Metric] = UNSET

    canceled_subscriptions_missing_features: OptionalNullable[Metric] = UNSET

    canceled_subscriptions_switched_service: OptionalNullable[Metric] = UNSET

    canceled_subscriptions_too_complex: OptionalNullable[Metric] = UNSET

    canceled_subscriptions_too_expensive: OptionalNullable[Metric] = UNSET

    canceled_subscriptions_unused: OptionalNullable[Metric] = UNSET

    canceled_subscriptions_other: OptionalNullable[Metric] = UNSET

    churned_subscriptions: OptionalNullable[Metric] = UNSET

    churn_rate: OptionalNullable[Metric] = UNSET

    ltv: OptionalNullable[Metric] = UNSET

    gross_margin: OptionalNullable[Metric] = UNSET

    gross_margin_percentage: OptionalNullable[Metric] = UNSET

    cashflow: OptionalNullable[Metric] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "orders",
            "revenue",
            "net_revenue",
            "cumulative_revenue",
            "net_cumulative_revenue",
            "costs",
            "cumulative_costs",
            "average_order_value",
            "net_average_order_value",
            "average_revenue_per_user",
            "cost_per_user",
            "active_user_by_event",
            "one_time_products",
            "one_time_products_revenue",
            "one_time_products_net_revenue",
            "new_subscriptions",
            "new_subscriptions_revenue",
            "new_subscriptions_net_revenue",
            "renewed_subscriptions",
            "renewed_subscriptions_revenue",
            "renewed_subscriptions_net_revenue",
            "active_subscriptions",
            "committed_subscriptions",
            "monthly_recurring_revenue",
            "committed_monthly_recurring_revenue",
            "checkouts",
            "succeeded_checkouts",
            "checkouts_conversion",
            "canceled_subscriptions",
            "canceled_subscriptions_customer_service",
            "canceled_subscriptions_low_quality",
            "canceled_subscriptions_missing_features",
            "canceled_subscriptions_switched_service",
            "canceled_subscriptions_too_complex",
            "canceled_subscriptions_too_expensive",
            "canceled_subscriptions_unused",
            "canceled_subscriptions_other",
            "churned_subscriptions",
            "churn_rate",
            "ltv",
            "gross_margin",
            "gross_margin_percentage",
            "cashflow",
        ]
        nullable_fields = [
            "orders",
            "revenue",
            "net_revenue",
            "cumulative_revenue",
            "net_cumulative_revenue",
            "costs",
            "cumulative_costs",
            "average_order_value",
            "net_average_order_value",
            "average_revenue_per_user",
            "cost_per_user",
            "active_user_by_event",
            "one_time_products",
            "one_time_products_revenue",
            "one_time_products_net_revenue",
            "new_subscriptions",
            "new_subscriptions_revenue",
            "new_subscriptions_net_revenue",
            "renewed_subscriptions",
            "renewed_subscriptions_revenue",
            "renewed_subscriptions_net_revenue",
            "active_subscriptions",
            "committed_subscriptions",
            "monthly_recurring_revenue",
            "committed_monthly_recurring_revenue",
            "checkouts",
            "succeeded_checkouts",
            "checkouts_conversion",
            "canceled_subscriptions",
            "canceled_subscriptions_customer_service",
            "canceled_subscriptions_low_quality",
            "canceled_subscriptions_missing_features",
            "canceled_subscriptions_switched_service",
            "canceled_subscriptions_too_complex",
            "canceled_subscriptions_too_expensive",
            "canceled_subscriptions_unused",
            "canceled_subscriptions_other",
            "churned_subscriptions",
            "churn_rate",
            "ltv",
            "gross_margin",
            "gross_margin_percentage",
            "cashflow",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .authorizeuser import AuthorizeUser, AuthorizeUserTypedDict
from .oauth2clientpublic import OAuth2ClientPublic, OAuth2ClientPublicTypedDict
from .scope import Scope
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Dict, List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AuthorizeResponseUserTypedDict(TypedDict):
    client: OAuth2ClientPublicTypedDict
    sub: Nullable[AuthorizeUserTypedDict]
    scopes: List[Scope]
    sub_type: Literal["user"]
    scope_display_names: NotRequired[Dict[str, str]]


class AuthorizeResponseUser(BaseModel):
    client: OAuth2ClientPublic

    sub: Nullable[AuthorizeUser]

    scopes: List[Scope]

    SUB_TYPE: Annotated[
        Annotated[Literal["user"], AfterValidator(validate_const("user"))],
        pydantic.Field(alias="sub_type"),
    ] = "user"

    scope_display_names: Optional[Dict[str, str]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["scope_display_names"]
        nullable_fields = ["sub"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .costmetadata_input import CostMetadataInput, CostMetadataInputTypedDict
from .llmmetadata import LLMMetadata, LLMMetadataTypedDict
from typing import Union
from typing_extensions import TypeAliasType


EventMetadataInputTypedDict = TypeAliasType(
    "EventMetadataInputTypedDict",
    Union[CostMetadataInputTypedDict, LLMMetadataTypedDict, str, int, float, bool],
)


EventMetadataInput = TypeAliasType(
    "EventMetadataInput", Union[CostMetadataInput, LLMMetadata, str, int, float, bool]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class SeatStatus(str, Enum):
    PENDING = "pending"
    CLAIMED = "claimed"
    REVOKED = "revoked"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerseat import CustomerSeat, CustomerSeatTypedDict
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CustomerSeatClaimResponseTypedDict(TypedDict):
    r"""Response after successfully claiming a seat."""

    seat: CustomerSeatTypedDict
    customer_session_token: str
    r"""Session token for immediate customer portal access"""


class CustomerSeatClaimResponse(BaseModel):
    r"""Response after successfully claiming a seat."""

    seat: CustomerSeat

    customer_session_token: str
    r"""Session token for immediate customer portal access"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class OrderPaidMetadataTypedDict(TypedDict):
    order_id: str
    amount: int
    product_id: NotRequired[str]
    billing_type: NotRequired[str]
    currency: NotRequired[str]
    net_amount: NotRequired[int]
    tax_amount: NotRequired[int]
    applied_balance_amount: NotRequired[int]
    discount_amount: NotRequired[int]
    discount_id: NotRequired[str]
    platform_fee: NotRequired[int]
    subscription_id: NotRequired[str]
    recurring_interval: NotRequired[str]
    recurring_interval_count: NotRequired[int]


class OrderPaidMetadata(BaseModel):
    order_id: str

    amount: int

    product_id: Optional[str] = None

    billing_type: Optional[str] = None

    currency: Optional[str] = None

    net_amount: Optional[int] = None

    tax_amount: Optional[int] = None

    applied_balance_amount: Optional[int] = None

    discount_amount: Optional[int] = None

    discount_id: Optional[str] = None

    platform_fee: Optional[int] = None

    subscription_id: Optional[str] = None

    recurring_interval: Optional[str] = None

    recurring_interval_count: Optional[int] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .webhookeventtype import WebhookEventType
from .webhookformat import WebhookFormat
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List
from typing_extensions import Annotated, NotRequired, TypedDict


class WebhookEndpointUpdateTypedDict(TypedDict):
    r"""Schema to update a webhook endpoint."""

    url: NotRequired[Nullable[str]]
    secret: NotRequired[Nullable[str]]
    format_: NotRequired[Nullable[WebhookFormat]]
    events: NotRequired[Nullable[List[WebhookEventType]]]
    enabled: NotRequired[Nullable[bool]]
    r"""Whether the webhook endpoint is enabled."""


class WebhookEndpointUpdate(BaseModel):
    r"""Schema to update a webhook endpoint."""

    url: OptionalNullable[str] = UNSET

    secret: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET

    format_: Annotated[
        OptionalNullable[WebhookFormat], pydantic.Field(alias="format")
    ] = UNSET

    events: OptionalNullable[List[WebhookEventType]] = UNSET

    enabled: OptionalNullable[bool] = UNSET
    r"""Whether the webhook endpoint is enabled."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["url", "secret", "format", "events", "enabled"]
        nullable_fields = ["url", "secret", "format", "events", "enabled"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class SubscriptionSeatsUpdatedMetadataTypedDict(TypedDict):
    subscription_id: str
    old_seats: int
    new_seats: int
    proration_behavior: str


class SubscriptionSeatsUpdatedMetadata(BaseModel):
    subscription_id: str

    old_seats: int

    new_seats: int

    proration_behavior: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .eventtypewithstats import EventTypeWithStats, EventTypeWithStatsTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceEventTypeWithStatsTypedDict(TypedDict):
    items: List[EventTypeWithStatsTypedDict]
    pagination: PaginationTypedDict


class ListResourceEventTypeWithStats(BaseModel):
    items: List[EventTypeWithStats]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .listresource_ import ListResource, ListResourceTypedDict
from .paymentsortproperty import PaymentSortProperty
from .paymentstatus import PaymentStatus
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Callable, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


PaymentsListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "PaymentsListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


PaymentsListQueryParamOrganizationIDFilter = TypeAliasType(
    "PaymentsListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


PaymentsListQueryParamCheckoutIDFilterTypedDict = TypeAliasType(
    "PaymentsListQueryParamCheckoutIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by checkout ID."""


PaymentsListQueryParamCheckoutIDFilter = TypeAliasType(
    "PaymentsListQueryParamCheckoutIDFilter", Union[str, List[str]]
)
r"""Filter by checkout ID."""


PaymentsListQueryParamOrderIDFilterTypedDict = TypeAliasType(
    "PaymentsListQueryParamOrderIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by order ID."""


PaymentsListQueryParamOrderIDFilter = TypeAliasType(
    "PaymentsListQueryParamOrderIDFilter", Union[str, List[str]]
)
r"""Filter by order ID."""


PaymentsListQueryParamStatusFilterTypedDict = TypeAliasType(
    "PaymentsListQueryParamStatusFilterTypedDict",
    Union[PaymentStatus, List[PaymentStatus]],
)
r"""Filter by payment status."""


PaymentsListQueryParamStatusFilter = TypeAliasType(
    "PaymentsListQueryParamStatusFilter", Union[PaymentStatus, List[PaymentStatus]]
)
r"""Filter by payment status."""


MethodFilterTypedDict = TypeAliasType("MethodFilterTypedDict", Union[str, List[str]])
r"""Filter by payment method."""


MethodFilter = TypeAliasType("MethodFilter", Union[str, List[str]])
r"""Filter by payment method."""


CustomerEmailFilterTypedDict = TypeAliasType(
    "CustomerEmailFilterTypedDict", Union[str, List[str]]
)
r"""Filter by customer email."""


CustomerEmailFilter = TypeAliasType("CustomerEmailFilter", Union[str, List[str]])
r"""Filter by customer email."""


class PaymentsListRequestTypedDict(TypedDict):
    organization_id: NotRequired[
        Nullable[PaymentsListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    checkout_id: NotRequired[Nullable[PaymentsListQueryParamCheckoutIDFilterTypedDict]]
    r"""Filter by checkout ID."""
    order_id: NotRequired[Nullable[PaymentsListQueryParamOrderIDFilterTypedDict]]
    r"""Filter by order ID."""
    status: NotRequired[Nullable[PaymentsListQueryParamStatusFilterTypedDict]]
    r"""Filter by payment status."""
    method: NotRequired[Nullable[MethodFilterTypedDict]]
    r"""Filter by payment method."""
    customer_email: NotRequired[Nullable[CustomerEmailFilterTypedDict]]
    r"""Filter by customer email."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[PaymentSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""


class PaymentsListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[PaymentsListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    checkout_id: Annotated[
        OptionalNullable[PaymentsListQueryParamCheckoutIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by checkout ID."""

    order_id: Annotated[
        OptionalNullable[PaymentsListQueryParamOrderIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by order ID."""

    status: Annotated[
        OptionalNullable[PaymentsListQueryParamStatusFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by payment status."""

    method: Annotated[
        OptionalNullable[MethodFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by payment method."""

    customer_email: Annotated[
        OptionalNullable[CustomerEmailFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by customer email."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[PaymentSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "organization_id",
            "checkout_id",
            "order_id",
            "status",
            "method",
            "customer_email",
            "page",
            "limit",
            "sorting",
        ]
        nullable_fields = [
            "organization_id",
            "checkout_id",
            "order_id",
            "status",
            "method",
            "customer_email",
            "sorting",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class PaymentsListResponseTypedDict(TypedDict):
    result: ListResourceTypedDict


class PaymentsListResponse(BaseModel):
    next: Callable[[], Optional[PaymentsListResponse]]

    result: ListResource



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class PaymentSortProperty(str, Enum):
    CREATED_AT = "created_at"
    MINUS_CREATED_AT = "-created_at"
    STATUS = "status"
    MINUS_STATUS = "-status"
    AMOUNT = "amount"
    MINUS_AMOUNT = "-amount"
    METHOD = "method"
    MINUS_METHOD = "-method"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class CustomersDeleteRequestTypedDict(TypedDict):
    id: str
    r"""The customer ID."""


class CustomersDeleteRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The customer ID."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .member import Member, MemberTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceMemberTypedDict(TypedDict):
    items: List[MemberTypedDict]
    pagination: PaginationTypedDict


class ListResourceMember(BaseModel):
    items: List[Member]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CostMetadataOutputTypedDict(TypedDict):
    amount: str
    r"""The amount in cents."""
    currency: str
    r"""The currency. Currently, only `usd` is supported."""


class CostMetadataOutput(BaseModel):
    amount: str
    r"""The amount in cents."""

    currency: str
    r"""The currency. Currently, only `usd` is supported."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CustomerPortalSubscriptionSettingsTypedDict(TypedDict):
    update_seats: bool
    update_plan: bool


class CustomerPortalSubscriptionSettings(BaseModel):
    update_seats: bool

    update_plan: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class SubscriptionCycledMetadataTypedDict(TypedDict):
    subscription_id: str
    product_id: NotRequired[str]
    amount: NotRequired[int]
    currency: NotRequired[str]
    recurring_interval: NotRequired[str]
    recurring_interval_count: NotRequired[int]


class SubscriptionCycledMetadata(BaseModel):
    subscription_id: str

    product_id: Optional[str] = None

    amount: Optional[int] = None

    currency: Optional[str] = None

    recurring_interval: Optional[str] = None

    recurring_interval_count: Optional[int] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CursorPaginationTypedDict(TypedDict):
    has_next_page: bool


class CursorPagination(BaseModel):
    has_next_page: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerbenefitgrantdiscordpropertiesupdate import (
    CustomerBenefitGrantDiscordPropertiesUpdate,
    CustomerBenefitGrantDiscordPropertiesUpdateTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class CustomerBenefitGrantDiscordUpdateTypedDict(TypedDict):
    properties: CustomerBenefitGrantDiscordPropertiesUpdateTypedDict
    benefit_type: Literal["discord"]


class CustomerBenefitGrantDiscordUpdate(BaseModel):
    properties: CustomerBenefitGrantDiscordPropertiesUpdate

    BENEFIT_TYPE: Annotated[
        Annotated[Literal["discord"], AfterValidator(validate_const("discord"))],
        pydantic.Field(alias="benefit_type"),
    ] = "discord"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpriceseattier import ProductPriceSeatTier, ProductPriceSeatTierTypedDict
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List
from typing_extensions import TypedDict


class ProductPriceSeatTiersOutputTypedDict(TypedDict):
    r"""List of pricing tiers for seat-based pricing.

    The minimum and maximum seat limits are derived from the tiers:
    - minimum_seats = first tier's min_seats
    - maximum_seats = last tier's max_seats (None for unlimited)
    """

    tiers: List[ProductPriceSeatTierTypedDict]
    r"""List of pricing tiers"""
    minimum_seats: int
    r"""Minimum number of seats required for purchase, derived from first tier."""
    maximum_seats: Nullable[int]
    r"""Maximum number of seats allowed for purchase, derived from last tier. None for unlimited."""


class ProductPriceSeatTiersOutput(BaseModel):
    r"""List of pricing tiers for seat-based pricing.

    The minimum and maximum seat limits are derived from the tiers:
    - minimum_seats = first tier's min_seats
    - maximum_seats = last tier's max_seats (None for unlimited)
    """

    tiers: List[ProductPriceSeatTier]
    r"""List of pricing tiers"""

    minimum_seats: int
    r"""Minimum number of seats required for purchase, derived from first tier."""

    maximum_seats: Nullable[int]
    r"""Maximum number of seats allowed for purchase, derived from last tier. None for unlimited."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["maximum_seats"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitgrantcustomwebhook import (
    BenefitGrantCustomWebhook,
    BenefitGrantCustomWebhookTypedDict,
)
from .benefitgrantdiscordwebhook import (
    BenefitGrantDiscordWebhook,
    BenefitGrantDiscordWebhookTypedDict,
)
from .benefitgrantdownloadableswebhook import (
    BenefitGrantDownloadablesWebhook,
    BenefitGrantDownloadablesWebhookTypedDict,
)
from .benefitgrantgithubrepositorywebhook import (
    BenefitGrantGitHubRepositoryWebhook,
    BenefitGrantGitHubRepositoryWebhookTypedDict,
)
from .benefitgrantlicensekeyswebhook import (
    BenefitGrantLicenseKeysWebhook,
    BenefitGrantLicenseKeysWebhookTypedDict,
)
from .benefitgrantmetercreditwebhook import (
    BenefitGrantMeterCreditWebhook,
    BenefitGrantMeterCreditWebhookTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


BenefitGrantWebhookTypedDict = TypeAliasType(
    "BenefitGrantWebhookTypedDict",
    Union[
        BenefitGrantDiscordWebhookTypedDict,
        BenefitGrantCustomWebhookTypedDict,
        BenefitGrantGitHubRepositoryWebhookTypedDict,
        BenefitGrantDownloadablesWebhookTypedDict,
        BenefitGrantLicenseKeysWebhookTypedDict,
        BenefitGrantMeterCreditWebhookTypedDict,
    ],
)


BenefitGrantWebhook = TypeAliasType(
    "BenefitGrantWebhook",
    Union[
        BenefitGrantDiscordWebhook,
        BenefitGrantCustomWebhook,
        BenefitGrantGitHubRepositoryWebhook,
        BenefitGrantDownloadablesWebhook,
        BenefitGrantLicenseKeysWebhook,
        BenefitGrantMeterCreditWebhook,
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


OrdersModelTypedDict = TypeAliasType("OrdersModelTypedDict", Union[int, float])


OrdersModel = TypeAliasType("OrdersModel", Union[int, float])


RevenueTypedDict = TypeAliasType("RevenueTypedDict", Union[int, float])


Revenue = TypeAliasType("Revenue", Union[int, float])


NetRevenueTypedDict = TypeAliasType("NetRevenueTypedDict", Union[int, float])


NetRevenue = TypeAliasType("NetRevenue", Union[int, float])


CumulativeRevenueTypedDict = TypeAliasType(
    "CumulativeRevenueTypedDict", Union[int, float]
)


CumulativeRevenue = TypeAliasType("CumulativeRevenue", Union[int, float])


NetCumulativeRevenueTypedDict = TypeAliasType(
    "NetCumulativeRevenueTypedDict", Union[int, float]
)


NetCumulativeRevenue = TypeAliasType("NetCumulativeRevenue", Union[int, float])


CostsTypedDict = TypeAliasType("CostsTypedDict", Union[int, float])


Costs = TypeAliasType("Costs", Union[int, float])


CumulativeCostsTypedDict = TypeAliasType("CumulativeCostsTypedDict", Union[int, float])


CumulativeCosts = TypeAliasType("CumulativeCosts", Union[int, float])


AverageOrderValueTypedDict = TypeAliasType(
    "AverageOrderValueTypedDict", Union[int, float]
)


AverageOrderValue = TypeAliasType("AverageOrderValue", Union[int, float])


NetAverageOrderValueTypedDict = TypeAliasType(
    "NetAverageOrderValueTypedDict", Union[int, float]
)


NetAverageOrderValue = TypeAliasType("NetAverageOrderValue", Union[int, float])


AverageRevenuePerUserTypedDict = TypeAliasType(
    "AverageRevenuePerUserTypedDict", Union[int, float]
)


AverageRevenuePerUser = TypeAliasType("AverageRevenuePerUser", Union[int, float])


CostPerUserTypedDict = TypeAliasType("CostPerUserTypedDict", Union[int, float])


CostPerUser = TypeAliasType("CostPerUser", Union[int, float])


ActiveUserByEventTypedDict = TypeAliasType(
    "ActiveUserByEventTypedDict", Union[int, float]
)


ActiveUserByEvent = TypeAliasType("ActiveUserByEvent", Union[int, float])


OneTimeProductsTypedDict = TypeAliasType("OneTimeProductsTypedDict", Union[int, float])


OneTimeProducts = TypeAliasType("OneTimeProducts", Union[int, float])


OneTimeProductsRevenueTypedDict = TypeAliasType(
    "OneTimeProductsRevenueTypedDict", Union[int, float]
)


OneTimeProductsRevenue = TypeAliasType("OneTimeProductsRevenue", Union[int, float])


OneTimeProductsNetRevenueTypedDict = TypeAliasType(
    "OneTimeProductsNetRevenueTypedDict", Union[int, float]
)


OneTimeProductsNetRevenue = TypeAliasType(
    "OneTimeProductsNetRevenue", Union[int, float]
)


NewSubscriptionsTypedDict = TypeAliasType(
    "NewSubscriptionsTypedDict", Union[int, float]
)


NewSubscriptions = TypeAliasType("NewSubscriptions", Union[int, float])


NewSubscriptionsRevenueTypedDict = TypeAliasType(
    "NewSubscriptionsRevenueTypedDict", Union[int, float]
)


NewSubscriptionsRevenue = TypeAliasType("NewSubscriptionsRevenue", Union[int, float])


NewSubscriptionsNetRevenueTypedDict = TypeAliasType(
    "NewSubscriptionsNetRevenueTypedDict", Union[int, float]
)


NewSubscriptionsNetRevenue = TypeAliasType(
    "NewSubscriptionsNetRevenue", Union[int, float]
)


RenewedSubscriptionsTypedDict = TypeAliasType(
    "RenewedSubscriptionsTypedDict", Union[int, float]
)


RenewedSubscriptions = TypeAliasType("RenewedSubscriptions", Union[int, float])


RenewedSubscriptionsRevenueTypedDict = TypeAliasType(
    "RenewedSubscriptionsRevenueTypedDict", Union[int, float]
)


RenewedSubscriptionsRevenue = TypeAliasType(
    "RenewedSubscriptionsRevenue", Union[int, float]
)


RenewedSubscriptionsNetRevenueTypedDict = TypeAliasType(
    "RenewedSubscriptionsNetRevenueTypedDict", Union[int, float]
)


RenewedSubscriptionsNetRevenue = TypeAliasType(
    "RenewedSubscriptionsNetRevenue", Union[int, float]
)


ActiveSubscriptionsTypedDict = TypeAliasType(
    "ActiveSubscriptionsTypedDict", Union[int, float]
)


ActiveSubscriptions = TypeAliasType("ActiveSubscriptions", Union[int, float])


CommittedSubscriptionsTypedDict = TypeAliasType(
    "CommittedSubscriptionsTypedDict", Union[int, float]
)


CommittedSubscriptions = TypeAliasType("CommittedSubscriptions", Union[int, float])


MonthlyRecurringRevenueTypedDict = TypeAliasType(
    "MonthlyRecurringRevenueTypedDict", Union[int, float]
)


MonthlyRecurringRevenue = TypeAliasType("MonthlyRecurringRevenue", Union[int, float])


CommittedMonthlyRecurringRevenueTypedDict = TypeAliasType(
    "CommittedMonthlyRecurringRevenueTypedDict", Union[int, float]
)


CommittedMonthlyRecurringRevenue = TypeAliasType(
    "CommittedMonthlyRecurringRevenue", Union[int, float]
)


CheckoutsModelTypedDict = TypeAliasType("CheckoutsModelTypedDict", Union[int, float])


CheckoutsModel = TypeAliasType("CheckoutsModel", Union[int, float])


SucceededCheckoutsTypedDict = TypeAliasType(
    "SucceededCheckoutsTypedDict", Union[int, float]
)


SucceededCheckouts = TypeAliasType("SucceededCheckouts", Union[int, float])


CheckoutsConversionTypedDict = TypeAliasType(
    "CheckoutsConversionTypedDict", Union[int, float]
)


CheckoutsConversion = TypeAliasType("CheckoutsConversion", Union[int, float])


CanceledSubscriptionsTypedDict = TypeAliasType(
    "CanceledSubscriptionsTypedDict", Union[int, float]
)


CanceledSubscriptions = TypeAliasType("CanceledSubscriptions", Union[int, float])


CanceledSubscriptionsCustomerServiceTypedDict = TypeAliasType(
    "CanceledSubscriptionsCustomerServiceTypedDict", Union[int, float]
)


CanceledSubscriptionsCustomerService = TypeAliasType(
    "CanceledSubscriptionsCustomerService", Union[int, float]
)


CanceledSubscriptionsLowQualityTypedDict = TypeAliasType(
    "CanceledSubscriptionsLowQualityTypedDict", Union[int, float]
)


CanceledSubscriptionsLowQuality = TypeAliasType(
    "CanceledSubscriptionsLowQuality", Union[int, float]
)


CanceledSubscriptionsMissingFeaturesTypedDict = TypeAliasType(
    "CanceledSubscriptionsMissingFeaturesTypedDict", Union[int, float]
)


CanceledSubscriptionsMissingFeatures = TypeAliasType(
    "CanceledSubscriptionsMissingFeatures", Union[int, float]
)


CanceledSubscriptionsSwitchedServiceTypedDict = TypeAliasType(
    "CanceledSubscriptionsSwitchedServiceTypedDict", Union[int, float]
)


CanceledSubscriptionsSwitchedService = TypeAliasType(
    "CanceledSubscriptionsSwitchedService", Union[int, float]
)


CanceledSubscriptionsTooComplexTypedDict = TypeAliasType(
    "CanceledSubscriptionsTooComplexTypedDict", Union[int, float]
)


CanceledSubscriptionsTooComplex = TypeAliasType(
    "CanceledSubscriptionsTooComplex", Union[int, float]
)


CanceledSubscriptionsTooExpensiveTypedDict = TypeAliasType(
    "CanceledSubscriptionsTooExpensiveTypedDict", Union[int, float]
)


CanceledSubscriptionsTooExpensive = TypeAliasType(
    "CanceledSubscriptionsTooExpensive", Union[int, float]
)


CanceledSubscriptionsUnusedTypedDict = TypeAliasType(
    "CanceledSubscriptionsUnusedTypedDict", Union[int, float]
)


CanceledSubscriptionsUnused = TypeAliasType(
    "CanceledSubscriptionsUnused", Union[int, float]
)


CanceledSubscriptionsOtherTypedDict = TypeAliasType(
    "CanceledSubscriptionsOtherTypedDict", Union[int, float]
)


CanceledSubscriptionsOther = TypeAliasType(
    "CanceledSubscriptionsOther", Union[int, float]
)


ChurnedSubscriptionsTypedDict = TypeAliasType(
    "ChurnedSubscriptionsTypedDict", Union[int, float]
)


ChurnedSubscriptions = TypeAliasType("ChurnedSubscriptions", Union[int, float])


ChurnRateTypedDict = TypeAliasType("ChurnRateTypedDict", Union[int, float])


ChurnRate = TypeAliasType("ChurnRate", Union[int, float])


LtvTypedDict = TypeAliasType("LtvTypedDict", Union[int, float])


Ltv = TypeAliasType("Ltv", Union[int, float])


GrossMarginTypedDict = TypeAliasType("GrossMarginTypedDict", Union[int, float])


GrossMargin = TypeAliasType("GrossMargin", Union[int, float])


GrossMarginPercentageTypedDict = TypeAliasType(
    "GrossMarginPercentageTypedDict", Union[int, float]
)


GrossMarginPercentage = TypeAliasType("GrossMarginPercentage", Union[int, float])


CashflowTypedDict = TypeAliasType("CashflowTypedDict", Union[int, float])


Cashflow = TypeAliasType("Cashflow", Union[int, float])


class MetricPeriodTypedDict(TypedDict):
    timestamp: datetime
    r"""Timestamp of this period data."""
    orders: NotRequired[Nullable[OrdersModelTypedDict]]
    revenue: NotRequired[Nullable[RevenueTypedDict]]
    net_revenue: NotRequired[Nullable[NetRevenueTypedDict]]
    cumulative_revenue: NotRequired[Nullable[CumulativeRevenueTypedDict]]
    net_cumulative_revenue: NotRequired[Nullable[NetCumulativeRevenueTypedDict]]
    costs: NotRequired[Nullable[CostsTypedDict]]
    cumulative_costs: NotRequired[Nullable[CumulativeCostsTypedDict]]
    average_order_value: NotRequired[Nullable[AverageOrderValueTypedDict]]
    net_average_order_value: NotRequired[Nullable[NetAverageOrderValueTypedDict]]
    average_revenue_per_user: NotRequired[Nullable[AverageRevenuePerUserTypedDict]]
    cost_per_user: NotRequired[Nullable[CostPerUserTypedDict]]
    active_user_by_event: NotRequired[Nullable[ActiveUserByEventTypedDict]]
    one_time_products: NotRequired[Nullable[OneTimeProductsTypedDict]]
    one_time_products_revenue: NotRequired[Nullable[OneTimeProductsRevenueTypedDict]]
    one_time_products_net_revenue: NotRequired[
        Nullable[OneTimeProductsNetRevenueTypedDict]
    ]
    new_subscriptions: NotRequired[Nullable[NewSubscriptionsTypedDict]]
    new_subscriptions_revenue: NotRequired[Nullable[NewSubscriptionsRevenueTypedDict]]
    new_subscriptions_net_revenue: NotRequired[
        Nullable[NewSubscriptionsNetRevenueTypedDict]
    ]
    renewed_subscriptions: NotRequired[Nullable[RenewedSubscriptionsTypedDict]]
    renewed_subscriptions_revenue: NotRequired[
        Nullable[RenewedSubscriptionsRevenueTypedDict]
    ]
    renewed_subscriptions_net_revenue: NotRequired[
        Nullable[RenewedSubscriptionsNetRevenueTypedDict]
    ]
    active_subscriptions: NotRequired[Nullable[ActiveSubscriptionsTypedDict]]
    committed_subscriptions: NotRequired[Nullable[CommittedSubscriptionsTypedDict]]
    monthly_recurring_revenue: NotRequired[Nullable[MonthlyRecurringRevenueTypedDict]]
    committed_monthly_recurring_revenue: NotRequired[
        Nullable[CommittedMonthlyRecurringRevenueTypedDict]
    ]
    checkouts: NotRequired[Nullable[CheckoutsModelTypedDict]]
    succeeded_checkouts: NotRequired[Nullable[SucceededCheckoutsTypedDict]]
    checkouts_conversion: NotRequired[Nullable[CheckoutsConversionTypedDict]]
    canceled_subscriptions: NotRequired[Nullable[CanceledSubscriptionsTypedDict]]
    canceled_subscriptions_customer_service: NotRequired[
        Nullable[CanceledSubscriptionsCustomerServiceTypedDict]
    ]
    canceled_subscriptions_low_quality: NotRequired[
        Nullable[CanceledSubscriptionsLowQualityTypedDict]
    ]
    canceled_subscriptions_missing_features: NotRequired[
        Nullable[CanceledSubscriptionsMissingFeaturesTypedDict]
    ]
    canceled_subscriptions_switched_service: NotRequired[
        Nullable[CanceledSubscriptionsSwitchedServiceTypedDict]
    ]
    canceled_subscriptions_too_complex: NotRequired[
        Nullable[CanceledSubscriptionsTooComplexTypedDict]
    ]
    canceled_subscriptions_too_expensive: NotRequired[
        Nullable[CanceledSubscriptionsTooExpensiveTypedDict]
    ]
    canceled_subscriptions_unused: NotRequired[
        Nullable[CanceledSubscriptionsUnusedTypedDict]
    ]
    canceled_subscriptions_other: NotRequired[
        Nullable[CanceledSubscriptionsOtherTypedDict]
    ]
    churned_subscriptions: NotRequired[Nullable[ChurnedSubscriptionsTypedDict]]
    churn_rate: NotRequired[Nullable[ChurnRateTypedDict]]
    ltv: NotRequired[Nullable[LtvTypedDict]]
    gross_margin: NotRequired[Nullable[GrossMarginTypedDict]]
    gross_margin_percentage: NotRequired[Nullable[GrossMarginPercentageTypedDict]]
    cashflow: NotRequired[Nullable[CashflowTypedDict]]


class MetricPeriod(BaseModel):
    timestamp: datetime
    r"""Timestamp of this period data."""

    orders: OptionalNullable[OrdersModel] = UNSET

    revenue: OptionalNullable[Revenue] = UNSET

    net_revenue: OptionalNullable[NetRevenue] = UNSET

    cumulative_revenue: OptionalNullable[CumulativeRevenue] = UNSET

    net_cumulative_revenue: OptionalNullable[NetCumulativeRevenue] = UNSET

    costs: OptionalNullable[Costs] = UNSET

    cumulative_costs: OptionalNullable[CumulativeCosts] = UNSET

    average_order_value: OptionalNullable[AverageOrderValue] = UNSET

    net_average_order_value: OptionalNullable[NetAverageOrderValue] = UNSET

    average_revenue_per_user: OptionalNullable[AverageRevenuePerUser] = UNSET

    cost_per_user: OptionalNullable[CostPerUser] = UNSET

    active_user_by_event: OptionalNullable[ActiveUserByEvent] = UNSET

    one_time_products: OptionalNullable[OneTimeProducts] = UNSET

    one_time_products_revenue: OptionalNullable[OneTimeProductsRevenue] = UNSET

    one_time_products_net_revenue: OptionalNullable[OneTimeProductsNetRevenue] = UNSET

    new_subscriptions: OptionalNullable[NewSubscriptions] = UNSET

    new_subscriptions_revenue: OptionalNullable[NewSubscriptionsRevenue] = UNSET

    new_subscriptions_net_revenue: OptionalNullable[NewSubscriptionsNetRevenue] = UNSET

    renewed_subscriptions: OptionalNullable[RenewedSubscriptions] = UNSET

    renewed_subscriptions_revenue: OptionalNullable[RenewedSubscriptionsRevenue] = UNSET

    renewed_subscriptions_net_revenue: OptionalNullable[
        RenewedSubscriptionsNetRevenue
    ] = UNSET

    active_subscriptions: OptionalNullable[ActiveSubscriptions] = UNSET

    committed_subscriptions: OptionalNullable[CommittedSubscriptions] = UNSET

    monthly_recurring_revenue: OptionalNullable[MonthlyRecurringRevenue] = UNSET

    committed_monthly_recurring_revenue: OptionalNullable[
        CommittedMonthlyRecurringRevenue
    ] = UNSET

    checkouts: OptionalNullable[CheckoutsModel] = UNSET

    succeeded_checkouts: OptionalNullable[SucceededCheckouts] = UNSET

    checkouts_conversion: OptionalNullable[CheckoutsConversion] = UNSET

    canceled_subscriptions: OptionalNullable[CanceledSubscriptions] = UNSET

    canceled_subscriptions_customer_service: OptionalNullable[
        CanceledSubscriptionsCustomerService
    ] = UNSET

    canceled_subscriptions_low_quality: OptionalNullable[
        CanceledSubscriptionsLowQuality
    ] = UNSET

    canceled_subscriptions_missing_features: OptionalNullable[
        CanceledSubscriptionsMissingFeatures
    ] = UNSET

    canceled_subscriptions_switched_service: OptionalNullable[
        CanceledSubscriptionsSwitchedService
    ] = UNSET

    canceled_subscriptions_too_complex: OptionalNullable[
        CanceledSubscriptionsTooComplex
    ] = UNSET

    canceled_subscriptions_too_expensive: OptionalNullable[
        CanceledSubscriptionsTooExpensive
    ] = UNSET

    canceled_subscriptions_unused: OptionalNullable[CanceledSubscriptionsUnused] = UNSET

    canceled_subscriptions_other: OptionalNullable[CanceledSubscriptionsOther] = UNSET

    churned_subscriptions: OptionalNullable[ChurnedSubscriptions] = UNSET

    churn_rate: OptionalNullable[ChurnRate] = UNSET

    ltv: OptionalNullable[Ltv] = UNSET

    gross_margin: OptionalNullable[GrossMargin] = UNSET

    gross_margin_percentage: OptionalNullable[GrossMarginPercentage] = UNSET

    cashflow: OptionalNullable[Cashflow] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "orders",
            "revenue",
            "net_revenue",
            "cumulative_revenue",
            "net_cumulative_revenue",
            "costs",
            "cumulative_costs",
            "average_order_value",
            "net_average_order_value",
            "average_revenue_per_user",
            "cost_per_user",
            "active_user_by_event",
            "one_time_products",
            "one_time_products_revenue",
            "one_time_products_net_revenue",
            "new_subscriptions",
            "new_subscriptions_revenue",
            "new_subscriptions_net_revenue",
            "renewed_subscriptions",
            "renewed_subscriptions_revenue",
            "renewed_subscriptions_net_revenue",
            "active_subscriptions",
            "committed_subscriptions",
            "monthly_recurring_revenue",
            "committed_monthly_recurring_revenue",
            "checkouts",
            "succeeded_checkouts",
            "checkouts_conversion",
            "canceled_subscriptions",
            "canceled_subscriptions_customer_service",
            "canceled_subscriptions_low_quality",
            "canceled_subscriptions_missing_features",
            "canceled_subscriptions_switched_service",
            "canceled_subscriptions_too_complex",
            "canceled_subscriptions_too_expensive",
            "canceled_subscriptions_unused",
            "canceled_subscriptions_other",
            "churned_subscriptions",
            "churn_rate",
            "ltv",
            "gross_margin",
            "gross_margin_percentage",
            "cashflow",
        ]
        nullable_fields = [
            "orders",
            "revenue",
            "net_revenue",
            "cumulative_revenue",
            "net_cumulative_revenue",
            "costs",
            "cumulative_costs",
            "average_order_value",
            "net_average_order_value",
            "average_revenue_per_user",
            "cost_per_user",
            "active_user_by_event",
            "one_time_products",
            "one_time_products_revenue",
            "one_time_products_net_revenue",
            "new_subscriptions",
            "new_subscriptions_revenue",
            "new_subscriptions_net_revenue",
            "renewed_subscriptions",
            "renewed_subscriptions_revenue",
            "renewed_subscriptions_net_revenue",
            "active_subscriptions",
            "committed_subscriptions",
            "monthly_recurring_revenue",
            "committed_monthly_recurring_revenue",
            "checkouts",
            "succeeded_checkouts",
            "checkouts_conversion",
            "canceled_subscriptions",
            "canceled_subscriptions_customer_service",
            "canceled_subscriptions_low_quality",
            "canceled_subscriptions_missing_features",
            "canceled_subscriptions_switched_service",
            "canceled_subscriptions_too_complex",
            "canceled_subscriptions_too_expensive",
            "canceled_subscriptions_unused",
            "canceled_subscriptions_other",
            "churned_subscriptions",
            "churn_rate",
            "ltv",
            "gross_margin",
            "gross_margin_percentage",
            "cashflow",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkoutcreatedmetadata import (
    CheckoutCreatedMetadata,
    CheckoutCreatedMetadataTypedDict,
)
from .customer import Customer, CustomerTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CheckoutCreatedEventTypedDict(TypedDict):
    r"""An event created by Polar when a checkout is created."""

    id: str
    r"""The ID of the object."""
    timestamp: datetime
    r"""The timestamp of the event."""
    organization_id: str
    r"""The ID of the organization owning the event."""
    customer_id: Nullable[str]
    r"""ID of the customer in your Polar organization associated with the event."""
    customer: Nullable[CustomerTypedDict]
    r"""The customer associated with the event."""
    external_customer_id: Nullable[str]
    r"""ID of the customer in your system associated with the event."""
    label: str
    r"""Human readable label of the event type."""
    metadata: CheckoutCreatedMetadataTypedDict
    child_count: NotRequired[int]
    r"""Number of direct child events linked to this event."""
    parent_id: NotRequired[Nullable[str]]
    r"""The ID of the parent event."""
    source: Literal["system"]
    r"""The source of the event. `system` events are created by Polar. `user` events are the one you create through our ingestion API."""
    name: Literal["checkout.created"]
    r"""The name of the event."""


class CheckoutCreatedEvent(BaseModel):
    r"""An event created by Polar when a checkout is created."""

    id: str
    r"""The ID of the object."""

    timestamp: datetime
    r"""The timestamp of the event."""

    organization_id: str
    r"""The ID of the organization owning the event."""

    customer_id: Nullable[str]
    r"""ID of the customer in your Polar organization associated with the event."""

    customer: Nullable[Customer]
    r"""The customer associated with the event."""

    external_customer_id: Nullable[str]
    r"""ID of the customer in your system associated with the event."""

    label: str
    r"""Human readable label of the event type."""

    metadata: CheckoutCreatedMetadata

    child_count: Optional[int] = 0
    r"""Number of direct child events linked to this event."""

    parent_id: OptionalNullable[str] = UNSET
    r"""The ID of the parent event."""

    SOURCE: Annotated[
        Annotated[Literal["system"], AfterValidator(validate_const("system"))],
        pydantic.Field(alias="source"),
    ] = "system"
    r"""The source of the event. `system` events are created by Polar. `user` events are the one you create through our ingestion API."""

    NAME: Annotated[
        Annotated[
            Literal["checkout.created"],
            AfterValidator(validate_const("checkout.created")),
        ],
        pydantic.Field(alias="name"),
    ] = "checkout.created"
    r"""The name of the event."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["child_count", "parent_id"]
        nullable_fields = [
            "customer_id",
            "customer",
            "external_customer_id",
            "parent_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerorganization import CustomerOrganization, CustomerOrganizationTypedDict
from .customerproduct import CustomerProduct, CustomerProductTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class CustomerOrganizationDataTypedDict(TypedDict):
    r"""Schema of an organization and related data for customer portal."""

    organization: CustomerOrganizationTypedDict
    products: List[CustomerProductTypedDict]


class CustomerOrganizationData(BaseModel):
    r"""Schema of an organization and related data for customer portal."""

    organization: CustomerOrganization

    products: List[CustomerProduct]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from polar_sdk import models, utils
from polar_sdk._hooks import HookContext
from polar_sdk.types import BaseModel, OptionalNullable, UNSET
from polar_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union, cast


class Seats(BaseSDK):
    def list_seats(
        self,
        *,
        security: Union[
            models.CustomerPortalSeatsListSeatsSecurity,
            models.CustomerPortalSeatsListSeatsSecurityTypedDict,
        ],
        subscription_id: OptionalNullable[str] = UNSET,
        order_id: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SeatsList:
        r"""List Seats

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param subscription_id: Subscription ID
        :param order_id: Order ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CustomerPortalSeatsListSeatsRequest(
            subscription_id=subscription_id,
            order_id=order_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/customer-portal/seats",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalSeatsListSeatsSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:seats:list_seats",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "403", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SeatsList, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def list_seats_async(
        self,
        *,
        security: Union[
            models.CustomerPortalSeatsListSeatsSecurity,
            models.CustomerPortalSeatsListSeatsSecurityTypedDict,
        ],
        subscription_id: OptionalNullable[str] = UNSET,
        order_id: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SeatsList:
        r"""List Seats

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param subscription_id: Subscription ID
        :param order_id: Order ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CustomerPortalSeatsListSeatsRequest(
            subscription_id=subscription_id,
            order_id=order_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/customer-portal/seats",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalSeatsListSeatsSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:seats:list_seats",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "403", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.SeatsList, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def assign_seat(
        self,
        *,
        security: Union[
            models.CustomerPortalSeatsAssignSeatSecurity,
            models.CustomerPortalSeatsAssignSeatSecurityTypedDict,
        ],
        request: Union[models.SeatAssign, models.SeatAssignTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerSeat:
        r"""Assign Seat

        :param security:
        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.SeatAssign)
        request = cast(models.SeatAssign, request)

        req = self._build_request(
            method="POST",
            path="/v1/customer-portal/seats",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalSeatsAssignSeatSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.SeatAssign
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:seats:assign_seat",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerSeat, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, ["400", "401", "403", "404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def assign_seat_async(
        self,
        *,
        security: Union[
            models.CustomerPortalSeatsAssignSeatSecurity,
            models.CustomerPortalSeatsAssignSeatSecurityTypedDict,
        ],
        request: Union[models.SeatAssign, models.SeatAssignTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerSeat:
        r"""Assign Seat

        :param security:
        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.SeatAssign)
        request = cast(models.SeatAssign, request)

        req = self._build_request_async(
            method="POST",
            path="/v1/customer-portal/seats",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalSeatsAssignSeatSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.SeatAssign
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:seats:assign_seat",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerSeat, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, ["400", "401", "403", "404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def revoke_seat(
        self,
        *,
        security: Union[
            models.CustomerPortalSeatsRevokeSeatSecurity,
            models.CustomerPortalSeatsRevokeSeatSecurityTypedDict,
        ],
        seat_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerSeat:
        r"""Revoke Seat

        :param security:
        :param seat_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CustomerPortalSeatsRevokeSeatRequest(
            seat_id=seat_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/v1/customer-portal/seats/{seat_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalSeatsRevokeSeatSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:seats:revoke_seat",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "403", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerSeat, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def revoke_seat_async(
        self,
        *,
        security: Union[
            models.CustomerPortalSeatsRevokeSeatSecurity,
            models.CustomerPortalSeatsRevokeSeatSecurityTypedDict,
        ],
        seat_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerSeat:
        r"""Revoke Seat

        :param security:
        :param seat_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CustomerPortalSeatsRevokeSeatRequest(
            seat_id=seat_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v1/customer-portal/seats/{seat_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalSeatsRevokeSeatSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:seats:revoke_seat",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "403", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerSeat, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def resend_invitation(
        self,
        *,
        security: Union[
            models.CustomerPortalSeatsResendInvitationSecurity,
            models.CustomerPortalSeatsResendInvitationSecurityTypedDict,
        ],
        seat_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerSeat:
        r"""Resend Invitation

        :param security:
        :param seat_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CustomerPortalSeatsResendInvitationRequest(
            seat_id=seat_id,
        )

        req = self._build_request(
            method="POST",
            path="/v1/customer-portal/seats/{seat_id}/resend",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalSeatsResendInvitationSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:seats:resend_invitation",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerSeat, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, ["400", "401", "403", "404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def resend_invitation_async(
        self,
        *,
        security: Union[
            models.CustomerPortalSeatsResendInvitationSecurity,
            models.CustomerPortalSeatsResendInvitationSecurityTypedDict,
        ],
        seat_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerSeat:
        r"""Resend Invitation

        :param security:
        :param seat_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CustomerPortalSeatsResendInvitationRequest(
            seat_id=seat_id,
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/customer-portal/seats/{seat_id}/resend",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalSeatsResendInvitationSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:seats:resend_invitation",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerSeat, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, ["400", "401", "403", "404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def list_claimed_subscriptions(
        self,
        *,
        security: Union[
            models.CustomerPortalSeatsListClaimedSubscriptionsSecurity,
            models.CustomerPortalSeatsListClaimedSubscriptionsSecurityTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.CustomerSubscription]:
        r"""List Claimed Subscriptions

        List all subscriptions where the authenticated customer has claimed a seat.

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request(
            method="GET",
            path="/v1/customer-portal/seats/subscriptions",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalSeatsListClaimedSubscriptionsSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:seats:list_claimed_subscriptions",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(List[models.CustomerSubscription], http_res)
        if utils.match_response(http_res, ["401", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def list_claimed_subscriptions_async(
        self,
        *,
        security: Union[
            models.CustomerPortalSeatsListClaimedSubscriptionsSecurity,
            models.CustomerPortalSeatsListClaimedSubscriptionsSecurityTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.CustomerSubscription]:
        r"""List Claimed Subscriptions

        List all subscriptions where the authenticated customer has claimed a seat.

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request_async(
            method="GET",
            path="/v1/customer-portal/seats/subscriptions",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalSeatsListClaimedSubscriptionsSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:seats:list_claimed_subscriptions",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(List[models.CustomerSubscription], http_res)
        if utils.match_response(http_res, ["401", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class BenefitGrantSortProperty(str, Enum):
    CREATED_AT = "created_at"
    MINUS_CREATED_AT = "-created_at"
    GRANTED_AT = "granted_at"
    MINUS_GRANTED_AT = "-granted_at"
    REVOKED_AT = "revoked_at"
    MINUS_REVOKED_AT = "-revoked_at"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class LLMMetadataTypedDict(TypedDict):
    vendor: str
    r"""The vendor of the event."""
    model: str
    r"""The model used for the event."""
    input_tokens: int
    r"""The number of LLM input tokens used for the event."""
    output_tokens: int
    r"""The number of LLM output tokens used for the event."""
    total_tokens: int
    r"""The total number of LLM tokens used for the event."""
    prompt: NotRequired[Nullable[str]]
    r"""The LLM prompt used for the event."""
    response: NotRequired[Nullable[str]]
    r"""The LLM response used for the event."""
    cached_input_tokens: NotRequired[int]
    r"""The number of LLM cached tokens that were used for the event."""


class LLMMetadata(BaseModel):
    vendor: str
    r"""The vendor of the event."""

    model: str
    r"""The model used for the event."""

    input_tokens: int
    r"""The number of LLM input tokens used for the event."""

    output_tokens: int
    r"""The number of LLM output tokens used for the event."""

    total_tokens: int
    r"""The total number of LLM tokens used for the event."""

    prompt: OptionalNullable[str] = UNSET
    r"""The LLM prompt used for the event."""

    response: OptionalNullable[str] = UNSET
    r"""The LLM response used for the event."""

    cached_input_tokens: Optional[int] = None
    r"""The number of LLM cached tokens that were used for the event."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["prompt", "response", "cached_input_tokens"]
        nullable_fields = ["prompt", "response"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerportalsubscriptionsettings import (
    CustomerPortalSubscriptionSettings,
    CustomerPortalSubscriptionSettingsTypedDict,
)
from .customerportalusagesettings import (
    CustomerPortalUsageSettings,
    CustomerPortalUsageSettingsTypedDict,
)
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class OrganizationCustomerPortalSettingsTypedDict(TypedDict):
    usage: CustomerPortalUsageSettingsTypedDict
    subscription: CustomerPortalSubscriptionSettingsTypedDict


class OrganizationCustomerPortalSettings(BaseModel):
    usage: CustomerPortalUsageSettings

    subscription: CustomerPortalSubscriptionSettings



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscription import Subscription, SubscriptionTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class WebhookSubscriptionCreatedPayloadTypedDict(TypedDict):
    r"""Sent when a new subscription is created.

    When this event occurs, the subscription `status` might not be `active` yet, as we can still have to wait for the first payment to be processed.

    **Discord & Slack support:** Full
    """

    timestamp: datetime
    data: SubscriptionTypedDict
    type: Literal["subscription.created"]


class WebhookSubscriptionCreatedPayload(BaseModel):
    r"""Sent when a new subscription is created.

    When this event occurs, the subscription `status` might not be `active` yet, as we can still have to wait for the first payment to be processed.

    **Discord & Slack support:** Full
    """

    timestamp: datetime

    data: Subscription

    TYPE: Annotated[
        Annotated[
            Literal["subscription.created"],
            AfterValidator(validate_const("subscription.created")),
        ],
        pydantic.Field(alias="type"),
    ] = "subscription.created"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .attachedcustomfield import AttachedCustomField, AttachedCustomFieldTypedDict
from .checkoutbillingaddressfields import (
    CheckoutBillingAddressFields,
    CheckoutBillingAddressFieldsTypedDict,
)
from .checkoutdiscountfixedonceforeverduration import (
    CheckoutDiscountFixedOnceForeverDuration,
    CheckoutDiscountFixedOnceForeverDurationTypedDict,
)
from .checkoutdiscountfixedrepeatduration import (
    CheckoutDiscountFixedRepeatDuration,
    CheckoutDiscountFixedRepeatDurationTypedDict,
)
from .checkoutdiscountpercentageonceforeverduration import (
    CheckoutDiscountPercentageOnceForeverDuration,
    CheckoutDiscountPercentageOnceForeverDurationTypedDict,
)
from .checkoutdiscountpercentagerepeatduration import (
    CheckoutDiscountPercentageRepeatDuration,
    CheckoutDiscountPercentageRepeatDurationTypedDict,
)
from .checkoutorganization import CheckoutOrganization, CheckoutOrganizationTypedDict
from .checkoutproduct import CheckoutProduct, CheckoutProductTypedDict
from .checkoutstatus import CheckoutStatus
from .legacyrecurringproductprice import (
    LegacyRecurringProductPrice,
    LegacyRecurringProductPriceTypedDict,
)
from .paymentprocessor import PaymentProcessor
from .productprice import ProductPrice, ProductPriceTypedDict
from .trialinterval import TrialInterval
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


CheckoutPublicCustomFieldDataTypedDict = TypeAliasType(
    "CheckoutPublicCustomFieldDataTypedDict", Union[str, int, bool, datetime]
)


CheckoutPublicCustomFieldData = TypeAliasType(
    "CheckoutPublicCustomFieldData", Union[str, int, bool, datetime]
)


CheckoutPublicProductPriceTypedDict = TypeAliasType(
    "CheckoutPublicProductPriceTypedDict",
    Union[LegacyRecurringProductPriceTypedDict, ProductPriceTypedDict],
)


CheckoutPublicProductPrice = TypeAliasType(
    "CheckoutPublicProductPrice", Union[LegacyRecurringProductPrice, ProductPrice]
)


CheckoutPublicPricesTypedDict = TypeAliasType(
    "CheckoutPublicPricesTypedDict",
    Union[LegacyRecurringProductPriceTypedDict, ProductPriceTypedDict],
)


CheckoutPublicPrices = TypeAliasType(
    "CheckoutPublicPrices", Union[LegacyRecurringProductPrice, ProductPrice]
)


CheckoutPublicDiscountTypedDict = TypeAliasType(
    "CheckoutPublicDiscountTypedDict",
    Union[
        CheckoutDiscountPercentageOnceForeverDurationTypedDict,
        CheckoutDiscountFixedOnceForeverDurationTypedDict,
        CheckoutDiscountPercentageRepeatDurationTypedDict,
        CheckoutDiscountFixedRepeatDurationTypedDict,
    ],
)


CheckoutPublicDiscount = TypeAliasType(
    "CheckoutPublicDiscount",
    Union[
        CheckoutDiscountPercentageOnceForeverDuration,
        CheckoutDiscountFixedOnceForeverDuration,
        CheckoutDiscountPercentageRepeatDuration,
        CheckoutDiscountFixedRepeatDuration,
    ],
)


class CheckoutPublicTypedDict(TypedDict):
    r"""Checkout session data retrieved using the client secret."""

    id: str
    r"""The ID of the object."""
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    payment_processor: PaymentProcessor
    status: CheckoutStatus
    client_secret: str
    r"""Client secret used to update and complete the checkout session from the client."""
    url: str
    r"""URL where the customer can access the checkout session."""
    expires_at: datetime
    r"""Expiration date and time of the checkout session."""
    success_url: str
    r"""URL where the customer will be redirected after a successful payment."""
    return_url: Nullable[str]
    r"""When set, a back button will be shown in the checkout to return to this URL."""
    embed_origin: Nullable[str]
    r"""When checkout is embedded, represents the Origin of the page embedding the checkout. Used as a security measure to send messages only to the embedding page."""
    amount: int
    r"""Amount in cents, before discounts and taxes."""
    discount_amount: int
    r"""Discount amount in cents."""
    net_amount: int
    r"""Amount in cents, after discounts but before taxes."""
    tax_amount: Nullable[int]
    r"""Sales tax amount in cents. If `null`, it means there is no enough information yet to calculate it."""
    total_amount: int
    r"""Amount in cents, after discounts and taxes."""
    currency: str
    r"""Currency code of the checkout session."""
    allow_trial: Nullable[bool]
    r"""Whether to enable the trial period for the checkout session. If `false`, the trial period will be disabled, even if the selected product has a trial configured."""
    active_trial_interval: Nullable[TrialInterval]
    r"""Interval unit of the trial period, if any. This value is either set from the checkout, if `trial_interval` is set, or from the selected product."""
    active_trial_interval_count: Nullable[int]
    r"""Number of interval units of the trial period, if any. This value is either set from the checkout, if `trial_interval_count` is set, or from the selected product."""
    trial_end: Nullable[datetime]
    r"""End date and time of the trial period, if any."""
    organization_id: str
    r"""ID of the organization owning the checkout session."""
    product_id: Nullable[str]
    r"""ID of the product to checkout."""
    product_price_id: Nullable[str]
    r"""ID of the product price to checkout."""
    discount_id: Nullable[str]
    r"""ID of the discount applied to the checkout."""
    allow_discount_codes: bool
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""
    require_billing_address: bool
    r"""Whether to require the customer to fill their full billing address, instead of just the country. Customers in the US will always be required to fill their full address, regardless of this setting. If you preset the billing address, this setting will be automatically set to `true`."""
    is_discount_applicable: bool
    r"""Whether the discount is applicable to the checkout. Typically, free and custom prices are not discountable."""
    is_free_product_price: bool
    r"""Whether the product price is free, regardless of discounts."""
    is_payment_required: bool
    r"""Whether the checkout requires payment, e.g. in case of free products or discounts that cover the total amount."""
    is_payment_setup_required: bool
    r"""Whether the checkout requires setting up a payment method, regardless of the amount, e.g. subscriptions that have first free cycles."""
    is_payment_form_required: bool
    r"""Whether the checkout requires a payment form, whether because of a payment or payment method setup."""
    customer_id: Nullable[str]
    is_business_customer: bool
    r"""Whether the customer is a business or an individual. If `true`, the customer will be required to fill their full billing address and billing name."""
    customer_name: Nullable[str]
    r"""Name of the customer."""
    customer_email: Nullable[str]
    r"""Email address of the customer."""
    customer_ip_address: Nullable[str]
    customer_billing_name: Nullable[str]
    customer_billing_address: Nullable[AddressTypedDict]
    customer_tax_id: Nullable[str]
    payment_processor_metadata: Dict[str, str]
    billing_address_fields: CheckoutBillingAddressFieldsTypedDict
    products: List[CheckoutProductTypedDict]
    r"""List of products available to select."""
    product: Nullable[CheckoutProductTypedDict]
    r"""Product selected to checkout."""
    product_price: Nullable[CheckoutPublicProductPriceTypedDict]
    r"""Price of the selected product."""
    prices: Nullable[Dict[str, List[CheckoutPublicPricesTypedDict]]]
    r"""Mapping of product IDs to their list of prices."""
    discount: Nullable[CheckoutPublicDiscountTypedDict]
    organization: CheckoutOrganizationTypedDict
    attached_custom_fields: Nullable[List[AttachedCustomFieldTypedDict]]
    custom_field_data: NotRequired[
        Dict[str, Nullable[CheckoutPublicCustomFieldDataTypedDict]]
    ]
    r"""Key-value object storing custom field values."""
    seats: NotRequired[Nullable[int]]
    r"""Number of seats for seat-based pricing."""
    price_per_seat: NotRequired[Nullable[int]]
    r"""Price per seat in cents for the current seat count, based on the applicable tier. Only relevant for seat-based pricing."""


class CheckoutPublic(BaseModel):
    r"""Checkout session data retrieved using the client secret."""

    id: str
    r"""The ID of the object."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    payment_processor: PaymentProcessor

    status: CheckoutStatus

    client_secret: str
    r"""Client secret used to update and complete the checkout session from the client."""

    url: str
    r"""URL where the customer can access the checkout session."""

    expires_at: datetime
    r"""Expiration date and time of the checkout session."""

    success_url: str
    r"""URL where the customer will be redirected after a successful payment."""

    return_url: Nullable[str]
    r"""When set, a back button will be shown in the checkout to return to this URL."""

    embed_origin: Nullable[str]
    r"""When checkout is embedded, represents the Origin of the page embedding the checkout. Used as a security measure to send messages only to the embedding page."""

    amount: int
    r"""Amount in cents, before discounts and taxes."""

    discount_amount: int
    r"""Discount amount in cents."""

    net_amount: int
    r"""Amount in cents, after discounts but before taxes."""

    tax_amount: Nullable[int]
    r"""Sales tax amount in cents. If `null`, it means there is no enough information yet to calculate it."""

    total_amount: int
    r"""Amount in cents, after discounts and taxes."""

    currency: str
    r"""Currency code of the checkout session."""

    allow_trial: Nullable[bool]
    r"""Whether to enable the trial period for the checkout session. If `false`, the trial period will be disabled, even if the selected product has a trial configured."""

    active_trial_interval: Nullable[TrialInterval]
    r"""Interval unit of the trial period, if any. This value is either set from the checkout, if `trial_interval` is set, or from the selected product."""

    active_trial_interval_count: Nullable[int]
    r"""Number of interval units of the trial period, if any. This value is either set from the checkout, if `trial_interval_count` is set, or from the selected product."""

    trial_end: Nullable[datetime]
    r"""End date and time of the trial period, if any."""

    organization_id: str
    r"""ID of the organization owning the checkout session."""

    product_id: Nullable[str]
    r"""ID of the product to checkout."""

    product_price_id: Annotated[
        Nullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]
    r"""ID of the product price to checkout."""

    discount_id: Nullable[str]
    r"""ID of the discount applied to the checkout."""

    allow_discount_codes: bool
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""

    require_billing_address: bool
    r"""Whether to require the customer to fill their full billing address, instead of just the country. Customers in the US will always be required to fill their full address, regardless of this setting. If you preset the billing address, this setting will be automatically set to `true`."""

    is_discount_applicable: bool
    r"""Whether the discount is applicable to the checkout. Typically, free and custom prices are not discountable."""

    is_free_product_price: bool
    r"""Whether the product price is free, regardless of discounts."""

    is_payment_required: bool
    r"""Whether the checkout requires payment, e.g. in case of free products or discounts that cover the total amount."""

    is_payment_setup_required: bool
    r"""Whether the checkout requires setting up a payment method, regardless of the amount, e.g. subscriptions that have first free cycles."""

    is_payment_form_required: bool
    r"""Whether the checkout requires a payment form, whether because of a payment or payment method setup."""

    customer_id: Nullable[str]

    is_business_customer: bool
    r"""Whether the customer is a business or an individual. If `true`, the customer will be required to fill their full billing address and billing name."""

    customer_name: Nullable[str]
    r"""Name of the customer."""

    customer_email: Nullable[str]
    r"""Email address of the customer."""

    customer_ip_address: Nullable[str]

    customer_billing_name: Nullable[str]

    customer_billing_address: Nullable[Address]

    customer_tax_id: Nullable[str]

    payment_processor_metadata: Dict[str, str]

    billing_address_fields: CheckoutBillingAddressFields

    products: List[CheckoutProduct]
    r"""List of products available to select."""

    product: Nullable[CheckoutProduct]
    r"""Product selected to checkout."""

    product_price: Annotated[
        Nullable[CheckoutPublicProductPrice],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]
    r"""Price of the selected product."""

    prices: Nullable[Dict[str, List[CheckoutPublicPrices]]]
    r"""Mapping of product IDs to their list of prices."""

    discount: Nullable[CheckoutPublicDiscount]

    organization: CheckoutOrganization

    attached_custom_fields: Nullable[List[AttachedCustomField]]

    custom_field_data: Optional[Dict[str, Nullable[CheckoutPublicCustomFieldData]]] = (
        None
    )
    r"""Key-value object storing custom field values."""

    seats: OptionalNullable[int] = UNSET
    r"""Number of seats for seat-based pricing."""

    price_per_seat: OptionalNullable[int] = UNSET
    r"""Price per seat in cents for the current seat count, based on the applicable tier. Only relevant for seat-based pricing."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["custom_field_data", "seats", "price_per_seat"]
        nullable_fields = [
            "modified_at",
            "return_url",
            "embed_origin",
            "seats",
            "price_per_seat",
            "tax_amount",
            "allow_trial",
            "active_trial_interval",
            "active_trial_interval_count",
            "trial_end",
            "product_id",
            "product_price_id",
            "discount_id",
            "customer_id",
            "customer_name",
            "customer_email",
            "customer_ip_address",
            "customer_billing_name",
            "customer_billing_address",
            "customer_tax_id",
            "product",
            "product_price",
            "prices",
            "discount",
            "attached_custom_fields",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .balancedisputeevent import BalanceDisputeEvent, BalanceDisputeEventTypedDict
from .balancedisputereversalevent import (
    BalanceDisputeReversalEvent,
    BalanceDisputeReversalEventTypedDict,
)
from .balanceorderevent import BalanceOrderEvent, BalanceOrderEventTypedDict
from .balancerefundevent import BalanceRefundEvent, BalanceRefundEventTypedDict
from .balancerefundreversalevent import (
    BalanceRefundReversalEvent,
    BalanceRefundReversalEventTypedDict,
)
from .benefitcycledevent import BenefitCycledEvent, BenefitCycledEventTypedDict
from .benefitgrantedevent import BenefitGrantedEvent, BenefitGrantedEventTypedDict
from .benefitrevokedevent import BenefitRevokedEvent, BenefitRevokedEventTypedDict
from .benefitupdatedevent import BenefitUpdatedEvent, BenefitUpdatedEventTypedDict
from .checkoutcreatedevent import CheckoutCreatedEvent, CheckoutCreatedEventTypedDict
from .customercreatedevent import CustomerCreatedEvent, CustomerCreatedEventTypedDict
from .customerdeletedevent import CustomerDeletedEvent, CustomerDeletedEventTypedDict
from .customerupdatedevent import CustomerUpdatedEvent, CustomerUpdatedEventTypedDict
from .metercreditevent import MeterCreditEvent, MeterCreditEventTypedDict
from .meterresetevent import MeterResetEvent, MeterResetEventTypedDict
from .orderpaidevent import OrderPaidEvent, OrderPaidEventTypedDict
from .orderrefundedevent import OrderRefundedEvent, OrderRefundedEventTypedDict
from .subscriptionbillingperiodupdatedevent import (
    SubscriptionBillingPeriodUpdatedEvent,
    SubscriptionBillingPeriodUpdatedEventTypedDict,
)
from .subscriptioncanceledevent import (
    SubscriptionCanceledEvent,
    SubscriptionCanceledEventTypedDict,
)
from .subscriptioncreatedevent import (
    SubscriptionCreatedEvent,
    SubscriptionCreatedEventTypedDict,
)
from .subscriptioncycledevent import (
    SubscriptionCycledEvent,
    SubscriptionCycledEventTypedDict,
)
from .subscriptionproductupdatedevent import (
    SubscriptionProductUpdatedEvent,
    SubscriptionProductUpdatedEventTypedDict,
)
from .subscriptionrevokedevent import (
    SubscriptionRevokedEvent,
    SubscriptionRevokedEventTypedDict,
)
from .subscriptionseatsupdatedevent import (
    SubscriptionSeatsUpdatedEvent,
    SubscriptionSeatsUpdatedEventTypedDict,
)
from .subscriptionuncanceledevent import (
    SubscriptionUncanceledEvent,
    SubscriptionUncanceledEventTypedDict,
)
from polar_sdk.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


SystemEventTypedDict = TypeAliasType(
    "SystemEventTypedDict",
    Union[
        MeterCreditEventTypedDict,
        MeterResetEventTypedDict,
        BenefitGrantedEventTypedDict,
        BenefitCycledEventTypedDict,
        BenefitUpdatedEventTypedDict,
        BenefitRevokedEventTypedDict,
        SubscriptionCreatedEventTypedDict,
        SubscriptionCycledEventTypedDict,
        SubscriptionCanceledEventTypedDict,
        SubscriptionRevokedEventTypedDict,
        SubscriptionUncanceledEventTypedDict,
        SubscriptionProductUpdatedEventTypedDict,
        SubscriptionSeatsUpdatedEventTypedDict,
        SubscriptionBillingPeriodUpdatedEventTypedDict,
        OrderPaidEventTypedDict,
        OrderRefundedEventTypedDict,
        CheckoutCreatedEventTypedDict,
        CustomerCreatedEventTypedDict,
        CustomerUpdatedEventTypedDict,
        CustomerDeletedEventTypedDict,
        BalanceOrderEventTypedDict,
        BalanceRefundEventTypedDict,
        BalanceRefundReversalEventTypedDict,
        BalanceDisputeEventTypedDict,
        BalanceDisputeReversalEventTypedDict,
    ],
)


SystemEvent = Annotated[
    Union[
        Annotated[BalanceDisputeEvent, Tag("balance.dispute")],
        Annotated[BalanceDisputeReversalEvent, Tag("balance.dispute_reversal")],
        Annotated[BalanceOrderEvent, Tag("balance.order")],
        Annotated[BalanceRefundEvent, Tag("balance.refund")],
        Annotated[BalanceRefundReversalEvent, Tag("balance.refund_reversal")],
        Annotated[BenefitCycledEvent, Tag("benefit.cycled")],
        Annotated[BenefitGrantedEvent, Tag("benefit.granted")],
        Annotated[BenefitRevokedEvent, Tag("benefit.revoked")],
        Annotated[BenefitUpdatedEvent, Tag("benefit.updated")],
        Annotated[CheckoutCreatedEvent, Tag("checkout.created")],
        Annotated[CustomerCreatedEvent, Tag("customer.created")],
        Annotated[CustomerDeletedEvent, Tag("customer.deleted")],
        Annotated[CustomerUpdatedEvent, Tag("customer.updated")],
        Annotated[MeterCreditEvent, Tag("meter.credited")],
        Annotated[MeterResetEvent, Tag("meter.reset")],
        Annotated[OrderPaidEvent, Tag("order.paid")],
        Annotated[OrderRefundedEvent, Tag("order.refunded")],
        Annotated[
            SubscriptionBillingPeriodUpdatedEvent,
            Tag("subscription.billing_period_updated"),
        ],
        Annotated[SubscriptionCanceledEvent, Tag("subscription.canceled")],
        Annotated[SubscriptionCreatedEvent, Tag("subscription.created")],
        Annotated[SubscriptionCycledEvent, Tag("subscription.cycled")],
        Annotated[SubscriptionProductUpdatedEvent, Tag("subscription.product_updated")],
        Annotated[SubscriptionRevokedEvent, Tag("subscription.revoked")],
        Annotated[SubscriptionSeatsUpdatedEvent, Tag("subscription.seats_updated")],
        Annotated[SubscriptionUncanceledEvent, Tag("subscription.uncanceled")],
    ],
    Discriminator(lambda m: get_discriminator(m, "name", "name")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class SubscriptionUpdateBillingPeriodTypedDict(TypedDict):
    current_billing_period_end: datetime
    r"""Set a new date for the end of the current billing period. The subscription will renew on this date. Needs to be later than the current value.

    It is not possible to update the current billing period on a canceled subscription.
    """


class SubscriptionUpdateBillingPeriod(BaseModel):
    current_billing_period_end: datetime
    r"""Set a new date for the end of the current billing period. The subscription will renew on this date. Needs to be later than the current value.

    It is not possible to update the current billing period on a canceled subscription.
    """



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dispute import Dispute, DisputeTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceDisputeTypedDict(TypedDict):
    items: List[DisputeTypedDict]
    pagination: PaginationTypedDict


class ListResourceDispute(BaseModel):
    items: List[Dispute]

    pagination: Pagination
